// src/components/h-cesium-viewer/useHCesiumViewerClsSatellite.ts
import type { HCesiumViewerCls, SatelliteOptions } from '@/components/h-cesium-viewer/h-cesium-viewer-class';
import type { MaybeRefOrGetter } from 'vue';

/**
 * 管理 Cesium Viewer 中的卫星实体，根据选中的 ID 列表进行同步。
 * @param hCesiumViewerInst - HCesiumViewerCls 实例或其 getter。
 * @param satelliteList - 包含所有可用卫星选项的数组或 getter。
 * @param selectedSatelliteIds - 包含当前选中卫星 ID 的数组或 getter。
 */
export function useHCesiumViewerClsSatellite(
  hCesiumViewerInst: MaybeRefOrGetter<HCesiumViewerCls | null>,
  satelliteList: MaybeRefOrGetter<Array<SatelliteOptions> | undefined>,
  selectedSatelliteIds: MaybeRefOrGetter<string[] | undefined>,
) {
  // 创建一个从 ID 到卫星选项的映射，方便查找
  const satelliteMap = computed(() => {
    const map = new Map<string, SatelliteOptions>();
    const list = toValue(satelliteList) ?? [];
    for (const satellite of list) {
      map.set(satellite.id, satellite);
    }
    return map;
  });

  // watchEffect: 同步 selectedSatelliteIds 到 Cesium 实体
  watchEffect(() => {
    const viewerInstance = toValue(hCesiumViewerInst);
    if (!viewerInstance) {
      // 如果 viewer 实例尚未初始化或已销毁，则不执行任何操作
      return;
    }

    const selectedIds = toValue(selectedSatelliteIds) ?? [];
    const selectedIdsSet = new Set(selectedIds); // 需要显示的卫星 ID
    const currentEntityIds = new Set(viewerInstance.currentSatelliteEntities.keys()); // 当前已显示的卫星 ID

    // 1. 移除不再选中的卫星
    for (const entityId of currentEntityIds) {
      if (!selectedIdsSet.has(entityId)) {
        // 如果当前显示的实体不在新的选中列表里，则移除
        viewerInstance.removeSatelliteById(entityId);
      }
    }

    // 2. 添加新增的选中卫星
    for (const selectedId of selectedIdsSet) {
      if (!currentEntityIds.has(selectedId)) {
        // 如果选中的 ID 对应的实体当前未显示，则添加
        const satelliteToAdd = satelliteMap.value.get(selectedId); // 从映射中查找卫星信息
        if (satelliteToAdd) {
          viewerInstance.addSatellite(satelliteToAdd);
        } else {
          // 如果在 satelliteList 中找不到对应的卫星信息，发出警告
          consola.warn(`无法找到 ID 为 "${selectedId}" 的卫星信息，无法添加到地图。`);
        }
      }
    }
  });

  // 返回 satelliteMap 可能在某些场景下有用
  return {
    satelliteMap,
  };
}
